import React, { forwardRef, useImperativeHandle } from 'react';

/** 工具 */
import { EMPTY_ARR, EMPTY_STR, MENU_TYPE_ENUM, LANG } from "./constants";
import { EditorContext } from "./util";

/** 小组件 */
import { Arrow } from "./components/Arrow";
import { Circle } from "./components/Circle";
import { Control } from "./components/Control";
import { Crop } from "./components/Crop";
import { Download } from "./components/Download";
import { Drag } from "./components/Drag";
import { Draw } from "./components/Draw";
import { Flip } from "./components/Flip";
import { History } from "./components/History";
import Mosaic from "./components/Mosaic";
import { Rect } from "./components/Rect";
import { Reset } from "./components/Reset";
import { Rotate } from "./components/Rotate";
import { Scale } from "./components/Scale";
import { Text } from "./components/Text";
import { Upload } from "./components/Upload";
import { useInit } from "./components/init";

/** 图像编辑器组件 */
export var Editor = /*#__PURE__*/forwardRef(function (_ref, ref) {
  var _ref$url = _ref.url,
    url = _ref$url === void 0 ? EMPTY_STR : _ref$url,
    style = _ref.style,
    _ref$menus = _ref.menus,
    menus = _ref$menus === void 0 ? EMPTY_ARR : _ref$menus,
    onDownLoad = _ref.onDownLoad,
    _ref$lang = _ref.lang,
    lang = _ref$lang === void 0 ? LANG.en : _ref$lang;
  var _useInit = useInit({
      url: url
    }),
    canvasEl = _useInit.canvasEl,
    canvasInstanceRef = _useInit.canvasInstanceRef,
    wrapperInstanceRef = _useInit.wrapperInstanceRef,
    currentMenu = _useInit.currentMenu,
    setCurrentMenu = _useInit.setCurrentMenu,
    currentMenuRef = _useInit.currentMenuRef,
    canvasIsRender = _useInit.canvasIsRender,
    initCanvasJson = _useInit.initCanvasJson,
    unSaveHistory = _useInit.unSaveHistory,
    historyRef = _useInit.historyRef,
    initBackGroundImage = _useInit.initBackGroundImage,
    downloadRef = _useInit.downloadRef,
    zoom = _useInit.zoom,
    setZoom = _useInit.setZoom;

  /** 将下载方法透出去 */
  useImperativeHandle(ref, function () {
    return {
      downloadRef: downloadRef
    };
  }, []);
  return /*#__PURE__*/React.createElement(EditorContext.Provider, {
    value: {
      canvasInstanceRef: canvasInstanceRef,
      wrapperInstanceRef: wrapperInstanceRef,
      currentMenuRef: currentMenuRef,
      currentMenu: currentMenu,
      setCurrentMenu: setCurrentMenu,
      canvasIsRender: canvasIsRender,
      canvasEl: canvasEl,
      url: url,
      onDownLoad: onDownLoad,
      initCanvasJson: initCanvasJson,
      unSaveHistory: unSaveHistory,
      historyRef: historyRef,
      initBackGroundImage: initBackGroundImage,
      zoom: zoom,
      setZoom: setZoom,
      lang: lang
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "tie-image-editor",
    style: style
  }, /*#__PURE__*/React.createElement(Control, null), /*#__PURE__*/React.createElement("div", {
    className: "tie-image-editor_tool clearfix"
  }, (!menus.length || menus.includes(MENU_TYPE_ENUM.rect)) && /*#__PURE__*/React.createElement(Rect, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.circle)) && /*#__PURE__*/React.createElement(Circle, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.arrow)) && /*#__PURE__*/React.createElement(Arrow, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.draw)) && /*#__PURE__*/React.createElement(Draw, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.text)) && /*#__PURE__*/React.createElement(Text, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.mosaic)) && /*#__PURE__*/React.createElement(Mosaic, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.crop)) && /*#__PURE__*/React.createElement(Crop, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.download)) && /*#__PURE__*/React.createElement(Download, {
    ref: downloadRef
  }), (!menus.length || menus.includes(MENU_TYPE_ENUM.upload)) && /*#__PURE__*/React.createElement(Upload, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.drag)) && /*#__PURE__*/React.createElement(Drag, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.reset)) && /*#__PURE__*/React.createElement(Reset, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.rotate)) && /*#__PURE__*/React.createElement(Rotate, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.flip)) && /*#__PURE__*/React.createElement(Flip, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.scale)) && /*#__PURE__*/React.createElement(Scale, null), (!menus.length || menus.includes(MENU_TYPE_ENUM.history)) && /*#__PURE__*/React.createElement(History, {
    ref: historyRef
  })), /*#__PURE__*/React.createElement("div", {
    ref: wrapperInstanceRef,
    className: "tie-image-editor_modal__wrapper"
  }, /*#__PURE__*/React.createElement("canvas", {
    ref: canvasEl
  }))));
});
Editor.displayName = 'Editor';