function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
/* eslint-disable @typescript-eslint/no-unused-expressions */
import classNames from 'classnames';
import React, { useContext, useEffect, useRef } from 'react';
import { ACTION, CURSOR, IPaintTypes, LANG, MENU_TYPE_ENUM, MENU_TYPE_TEXT, paintConfig } from "../constants";
import { EditorContext } from "../util";
import Paint from "./setting/Paint";
import Popover from "./setting/Popover";
import ArrowClass from "../helpers/Arrow";
export var useArrow = function useArrow() {
  var _useContext = useContext(EditorContext),
    currentMenu = _useContext.currentMenu,
    setCurrentMenu = _useContext.setCurrentMenu,
    canvasInstanceRef = _useContext.canvasInstanceRef,
    canvasIsRender = _useContext.canvasIsRender,
    currentMenuRef = _useContext.currentMenuRef,
    historyRef = _useContext.historyRef;
  var isDrawingLine = useRef(false);
  var pointer = useRef({
    x: 0,
    y: 0
  });
  var pointerPoints = useRef([]);
  var lineToDraw = useRef(null);
  var paintConfigValue = useRef({
    color: paintConfig.colors[0],
    size: paintConfig.sizes[0]
  });
  var initArrow = function initArrow() {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    canvas.on('mouse:down', function (o) {
      if (canvas.getActiveObject() || currentMenuRef.current !== MENU_TYPE_ENUM.arrow) {
        return;
      }
      canvas.discardActiveObject();
      canvas.getObjects().forEach(function (obj) {
        obj.selectable = false;
        obj.hasControls = false;
      });
      canvas.requestRenderAll();
      isDrawingLine.current = true;
      pointer.current = canvas.getPointer(o.e);
      pointerPoints.current = [pointer.current.x, pointer.current.y, pointer.current.x, pointer.current.y];
      var lineToDrawNew = new ArrowClass(pointerPoints.current, {
        strokeWidth: paintConfigValue.current.size,
        stroke: paintConfigValue.current.color,
        id: new Date().valueOf() + '_' + Math.random() * 4
      });

      // lineToDrawNew.selectable = false;
      // lineToDrawNew.evented = false;
      lineToDrawNew.strokeUniform = true;
      lineToDraw.current = lineToDrawNew;
      canvas.add(lineToDrawNew);
    });
    canvas.on('mouse:move', function (o) {
      if (!canvas.getActiveObject() && currentMenuRef.current === MENU_TYPE_ENUM.rect) {
        canvas.setCursor(CURSOR.crosshair);
      }
      if (canvas.getActiveObject() || currentMenuRef.current !== MENU_TYPE_ENUM.arrow || !isDrawingLine.current || !lineToDraw.current) {
        canvas.renderAll();
        return;
      }
      pointer.current = canvas.getPointer(o.e);
      if (o.e.shiftKey) {
        // calc angle
        var startX = pointerPoints.current[0];
        var startY = pointerPoints.current[1];
        var x2 = pointer.current.x - startX;
        var y2 = pointer.current.y - startY;
        var r = Math.sqrt(x2 * x2 + y2 * y2);
        var angle = Math.atan2(y2, x2) / Math.PI * 180;
        angle = parseInt('' + (angle + 7.5) % 360 / 15) * 15;
        var cosx = r * Math.cos(angle * Math.PI / 180);
        var sinx = r * Math.sin(angle * Math.PI / 180);
        lineToDraw.current.set({
          x2: cosx + startX,
          y2: sinx + startY
        });
      } else {
        if (!lineToDraw.current) {
          return;
        }
        lineToDraw.current.set({
          x2: pointer.current.x,
          y2: pointer.current.y
        });
      }
      canvas.renderAll();
    });
    canvas.on('mouse:up', function () {
      if (currentMenuRef.current !== MENU_TYPE_ENUM.arrow || !lineToDraw.current || !isDrawingLine.current) return;
      lineToDraw.current.setCoords();
      isDrawingLine.current = false;
      // canvas.discardActiveObject();

      if (pointer.current.x === pointerPoints.current[0] && pointer.current.y === pointerPoints.current[1]) {
        /** 无效矩形 */
        canvas.remove(lineToDraw.current);
        canvas.requestRenderAll();
        historyRef.current.updateCanvasState(ACTION.add, true, false);
      } else {
        /** 有效矩形 */
        canvas.setActiveObject(lineToDraw.current);
        historyRef.current.updateCanvasState(ACTION.add, true, true);
      }
    });
  };
  useEffect(function () {
    if (canvasInstanceRef.current && canvasIsRender) {
      initArrow();
    }
  }, [canvasInstanceRef, canvasIsRender]);
  var handleArrowTrigger = function handleArrowTrigger() {
    setCurrentMenu(currentMenu === MENU_TYPE_ENUM.arrow ? MENU_TYPE_ENUM.default : MENU_TYPE_ENUM.arrow);
  };
  var handlePaintChange = function handlePaintChange(type, value) {
    if (!canvasInstanceRef.current) {
      return;
    }
    var canvas = canvasInstanceRef.current;
    var activeObject = canvas.getActiveObjects()[0];
    if (type === IPaintTypes.color) {
      paintConfigValue.current.color = value;
      activeObject && activeObject.set('stroke', value);
    } else {
      paintConfigValue.current.size = value;
      activeObject && activeObject.set('strokeWidth', value);
    }
    canvas.renderAll();
  };
  return {
    handleArrowTrigger: handleArrowTrigger,
    handlePaintChange: handlePaintChange,
    currentMenu: currentMenu
  };
};

/** 箭头 */
export var Arrow = function Arrow() {
  var _useContext2 = useContext(EditorContext),
    _useContext2$lang = _useContext2.lang,
    lang = _useContext2$lang === void 0 ? LANG.en : _useContext2$lang;
  var _useArrow = useArrow(),
    handleArrowTrigger = _useArrow.handleArrowTrigger,
    handlePaintChange = _useArrow.handlePaintChange,
    currentMenu = _useArrow.currentMenu;
  return /*#__PURE__*/React.createElement("div", {
    className: classNames('tie-image-editor_tool-item tie-image-editor_tool-arrow', _defineProperty({}, 'tie-image-editor_tool-item--checked', currentMenu === MENU_TYPE_ENUM.arrow))
  }, /*#__PURE__*/React.createElement(Paint, {
    open: currentMenu === MENU_TYPE_ENUM.arrow,
    onChange: handlePaintChange
  }, /*#__PURE__*/React.createElement(Popover, {
    content: MENU_TYPE_TEXT.arrow[lang],
    placement: "top"
  }, /*#__PURE__*/React.createElement("i", {
    className: classNames('tie-image-editor_icon'),
    onClick: handleArrowTrigger
  }))));
};