function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
import classNames from 'classnames';
import { fabric } from 'fabric';
import React, { useCallback, useContext, useEffect, useRef } from 'react';
import { ACTION, IMAGE_NAME, LANG, MENU_TYPE_ENUM, MENU_TYPE_TEXT, WORK_SPACE_ID, keyCodes } from "../constants";
import Cropzone from "../helpers/CropZone";
import { EditorContext, clamp } from "../util";
import CropPop from "./setting/CropPop";
import Popover from "./setting/Popover";
var MOUSE_MOVE_THRESHOLD = 10;
export var useCrop = function useCrop() {
  var _useContext = useContext(EditorContext),
    canvasInstanceRef = _useContext.canvasInstanceRef,
    currentMenuRef = _useContext.currentMenuRef,
    setCurrentMenu = _useContext.setCurrentMenu,
    canvasIsRender = _useContext.canvasIsRender,
    currentMenu = _useContext.currentMenu,
    wrapperInstanceRef = _useContext.wrapperInstanceRef,
    historyRef = _useContext.historyRef,
    unSaveHistory = _useContext.unSaveHistory;

  /** 裁剪框实例 */
  var cropzone = useRef(null);
  /** 裁剪开始X */
  var startX = useRef(0);
  /** 裁剪开始Y */
  var startY = useRef(0);
  /** 是否有ShiftKey */
  var withShiftKey = useRef(false);

  /** 计算裁剪框坐标 */
  var calcRectDimensionFromPoint = useCallback(function (x, y) {
    var presetRatio = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    var canvasWidth = canvas.getWidth();
    var canvasHeight = canvas.getHeight();
    var startXNew = startX.current;
    var startYNew = startY.current;
    var left = clamp(x, 0, startXNew);
    var top = clamp(y, 0, startYNew);
    var width = clamp(x, startXNew, canvasWidth) - left; // 开始坐标小于鼠标坐标小于画布坐标
    var height = clamp(y, startYNew, canvasHeight) - top; //

    if (withShiftKey.current && !presetRatio) {
      if (width > height) {
        height = width;
      } else if (height > width) {
        width = height;
      }
      if (startXNew >= x) {
        left = startXNew - width;
      }
      if (startYNew >= y) {
        top = startYNew - height;
      }
    } else if (presetRatio) {
      height = width / presetRatio;
      if (startXNew >= x) {
        left = clamp(startXNew - width, 0, canvasWidth);
      }
      if (startYNew >= y) {
        top = clamp(startYNew - height, 0, canvasHeight);
      }
      if (top + height > canvasHeight) {
        height = canvasHeight - top;
        width = height * presetRatio;
        if (startXNew >= x) {
          left = clamp(startXNew - width, 0, canvasWidth);
        }
        if (startYNew >= y) {
          top = clamp(startYNew - height, 0, canvasHeight);
        }
      }
    }
    return {
      left: left,
      top: top,
      width: width,
      height: height
    };
  }, []);
  var keydown = useCallback(function (e) {
    if (currentMenuRef.current !== MENU_TYPE_ENUM.crop) {
      return;
    }
    if (e.keyCode === keyCodes.SHIFT) {
      withShiftKey.current = true;
    }
  }, []);
  var keyup = useCallback(function (e) {
    if (currentMenuRef.current !== MENU_TYPE_ENUM.crop) {
      return;
    }
    if (e.keyCode === keyCodes.SHIFT) {
      withShiftKey.current = false;
    }
  }, []);
  var mousemove = useCallback(function (fEvent) {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    if (currentMenuRef.current !== MENU_TYPE_ENUM.crop) {
      return;
    }
    var pointer = canvas.getPointer(fEvent.e);
    var x = pointer.x,
      y = pointer.y;
    var cropzoneNew = cropzone.current;
    if (!cropzoneNew) {
      return;
    }
    if (Math.abs(x - startX.current) + Math.abs(y - startY.current) > MOUSE_MOVE_THRESHOLD) {
      canvas.remove(cropzoneNew);
      cropzoneNew.set(calcRectDimensionFromPoint(x, y, cropzoneNew.presetRatio));
      canvas.add(cropzoneNew);
      canvas.setActiveObject(cropzoneNew);
    }
  }, [canvasIsRender]);
  var mouseup = useCallback(function () {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    if (currentMenuRef.current !== MENU_TYPE_ENUM.crop) {
      return;
    }
    var cropzoneNew = cropzone.current;
    canvas.setActiveObject(cropzoneNew);
    canvas.off('mouse:move', mousemove);
    canvas.off('mouse:up', mouseup);
  }, [canvasIsRender]);
  var mousedown = useCallback(function (fEvent) {
    if (fEvent.target) {
      return;
    }
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    if (currentMenuRef.current !== MENU_TYPE_ENUM.crop) {
      return;
    }
    canvas.selection = false;
    var coord = canvas.getPointer(fEvent.e);
    startX.current = coord.x;
    startY.current = coord.y;
    canvas.on('mouse:move', mousemove);
    canvas.on('mouse:up', mouseup);
  }, [canvasIsRender]);
  var start = useCallback(function () {
    var canvas = canvasInstanceRef.current;
    if (!canvas || !canvasIsRender) {
      return;
    }
    if (cropzone.current) {
      return;
    }

    /** 不让触发ObjectAdd事件 */
    unSaveHistory.current = true;
    canvas.getObjects().forEach(function (obj) {
      obj.evented = false;
    });
    cropzone.current = new Cropzone(canvasInstanceRef.current, {
      left: 0,
      top: 0,
      width: 0.5,
      height: 0.5,
      strokeWidth: 0,
      cornerSize: 16,
      cornerColor: 'blue',
      fill: 'transparent',
      hasRotatingPoint: false,
      hasBorders: false,
      lockScalingFlip: true,
      lockRotation: true,
      lockSkewingX: true,
      lockSkewingY: true,
      cornerStyle: 'circle',
      cornerStrokeColor: 'blue',
      transparentCorners: false,
      lineWidth: 2,
      borderColor: 'blue'
    });
    canvas.discardActiveObject();
    canvas.add(cropzone.current);
    canvas.on('mouse:down', mousedown);
    canvas.selection = false;
    canvas.defaultCursor = 'crosshair';
    fabric.util.addListener(document, 'keydown', keydown);
    fabric.util.addListener(document, 'keyup', keyup);
  }, [canvasIsRender]);
  var end = useCallback(function () {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    var cropzoneNew = cropzone.current;
    if (!cropzoneNew) {
      return;
    }
    canvas.remove(cropzoneNew);
    canvas.selection = true;
    canvas.defaultCursor = 'default';
    canvas.off('mouse:down', mousedown);
    canvas.off('mouse:move', mousemove);
    canvas.off('mouse:up', mouseup);
    canvas.forEachObject(function (obj) {
      obj.evented = true;
    });
    cropzone.current = null;
    fabric.util.removeListener(document, 'keydown', keydown);
    fabric.util.removeListener(document, 'keyup', keyup);
    /** 解除，可以触发ObjectAdd事件 */
    unSaveHistory.current = false;
  }, [canvasIsRender]);

  /** 切换裁剪 */
  var handleCropTrigger = function handleCropTrigger() {
    if (currentMenuRef.current !== MENU_TYPE_ENUM.crop) {
      start();
    } else {
      end();
    }
    setCurrentMenu(currentMenuRef.current !== MENU_TYPE_ENUM.crop ? MENU_TYPE_ENUM.crop : MENU_TYPE_ENUM.default);
  };

  /** 获取裁剪框参数 */
  var getCropzoneRect = function getCropzoneRect() {
    var cropzoneNew = cropzone.current;
    if (!cropzoneNew || !cropzoneNew.isValid()) {
      return null;
    }
    return {
      left: cropzoneNew.left,
      top: cropzoneNew.top,
      width: cropzoneNew.width,
      height: cropzoneNew.height
    };
  };

  /** 获取裁剪部分图片数据 */
  var getCroppedImageData = function getCroppedImageData() {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    var containsCropzone = canvas.contains(cropzone.current);
    var cropRect = getCropzoneRect();
    if (!cropRect) {
      return null;
    }
    if (containsCropzone) {
      canvas.remove(cropzone.current);
    }
    var imageData = {
      imageName: IMAGE_NAME,
      url: canvas.toDataURL(cropRect)
    };
    if (containsCropzone) {
      canvas.add(cropzone.current);
    }
    return imageData;
  };

  /** 确认或取消裁剪 */
  var handleCropChange = function handleCropChange(value) {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    if (!value) {
      end();
      start();
    } else {
      var data = getCroppedImageData();
      if (!data) {
        return;
      }
      if (!data.url.includes('base64')) {
        // Toast.warn('请划分裁剪区域', 1500);
        return;
      }
      canvas.clear();
      canvas.setBackgroundImage(data.url, function () {
        var oImg = canvas.backgroundImage;
        var _wrapperInstanceRef$c = wrapperInstanceRef.current.getBoundingClientRect(),
          height = _wrapperInstanceRef$c.height;
        var center = canvas.getCenter();
        var imgHeight = oImg.height;
        /** 图片过大，使其大小正好跟容器一致 */
        oImg.scale(height / imgHeight);
        /** 使得图片在canvas中间 */
        oImg.set({
          left: center.left - oImg.width * (height / imgHeight) / 2,
          top: center.top - oImg.height * (height / imgHeight) / 2
        });
        /** 不让直接操作图片 */
        oImg.selectable = false;
        oImg.id = WORK_SPACE_ID;
        canvas.renderAll();

        /** 准备下一次裁剪 */
        end();
        historyRef.current.updateCanvasState(ACTION.add);
        start();
      });
    }
  };
  useEffect(function () {
    if (currentMenu !== MENU_TYPE_ENUM.crop && cropzone.current) {
      /** 切换到其他菜单时清空 */
      end();
    }
  }, [currentMenu]);
  return {
    handleCropTrigger: handleCropTrigger,
    handleCropChange: handleCropChange,
    currentMenu: currentMenu
  };
};

/** 裁剪 */
export var Crop = function Crop() {
  var _useContext2 = useContext(EditorContext),
    _useContext2$lang = _useContext2.lang,
    lang = _useContext2$lang === void 0 ? LANG.en : _useContext2$lang;
  var _useCrop = useCrop(),
    handleCropTrigger = _useCrop.handleCropTrigger,
    handleCropChange = _useCrop.handleCropChange,
    currentMenu = _useCrop.currentMenu;
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: classNames('tie-image-editor_tool-item tie-image-editor_tool-crop', _defineProperty({}, 'tie-image-editor_tool-item--checked', currentMenu === MENU_TYPE_ENUM.crop))
  }, /*#__PURE__*/React.createElement(CropPop, {
    open: currentMenu === MENU_TYPE_ENUM.crop,
    onChange: handleCropChange
  }, /*#__PURE__*/React.createElement(Popover, {
    content: MENU_TYPE_TEXT.crop[lang],
    placement: "top"
  }, /*#__PURE__*/React.createElement("i", {
    className: "tie-image-editor_icon",
    onClick: handleCropTrigger
  })))));
};