import React, { forwardRef, useContext, useImperativeHandle } from 'react';
import { IMAGE_NAME, LANG, MENU_TYPE_TEXT } from "../constants";
import { EditorContext, base64ToBlob, isSupportFileApi } from "../util";
import Popover from "./setting/Popover";
var useDownload = function useDownload() {
  var _useContext = useContext(EditorContext),
    canvasInstanceRef = _useContext.canvasInstanceRef,
    canvasEl = _useContext.canvasEl,
    onDownLoad = _useContext.onDownLoad;
  var handleDownload = function handleDownload() {
    var auto = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    console.log('heheh');
    var canvas = canvasInstanceRef.current;
    if (!canvas || !canvasEl.current) {
      return Promise.reject();
    }
    var dataURL = canvas.toDataURL();
    var imageName = Date.now() + IMAGE_NAME;
    var blob, type;
    if (onDownLoad) {
      onDownLoad({
        downLoadUrl: dataURL
      });
      return Promise.resolve({
        downLoadUrl: dataURL
      });
    }
    if (!auto) {
      return Promise.resolve({
        downLoadUrl: dataURL
      });
    }
    if (isSupportFileApi() && window.saveAs) {
      blob = base64ToBlob(dataURL);
      type = blob.type.split('/')[1];
      if (imageName.split('.').pop() !== type) {
        imageName += ".".concat(type);
      }
      window.saveAs(blob, imageName);
    } else {
      if (imageName.split('.').pop() !== type) {
        imageName += ".png";
      }
      var anchorEl = document.createElement('a');
      anchorEl.href = dataURL;
      anchorEl.download = imageName;
      document.body.appendChild(anchorEl);
      anchorEl.click();
      anchorEl.remove();
    }
    return Promise.resolve({
      downLoadUrl: dataURL
    });
  };
  return {
    handleDownload: handleDownload
  };
};

/** 下载 */
export var Download = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext2 = useContext(EditorContext),
    _useContext2$lang = _useContext2.lang,
    lang = _useContext2$lang === void 0 ? LANG.en : _useContext2$lang;
  var _useDownload = useDownload(),
    handleDownload = _useDownload.handleDownload;
  useImperativeHandle(ref, function () {
    return {
      download: handleDownload
    };
  }, [handleDownload]);
  return /*#__PURE__*/React.createElement("div", {
    className: "tie-image-editor_tool-item tie-image-editor_tool-download"
  }, /*#__PURE__*/React.createElement(Popover, {
    content: MENU_TYPE_TEXT.download[lang],
    placement: "top"
  }, /*#__PURE__*/React.createElement("i", {
    className: "tie-image-editor_icon",
    onClick: function onClick() {
      return handleDownload(true);
    }
  })));
});
Download.displayName = 'Download';