function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i]; return arr2; }
function _iterableToArrayLimit(r, l) { var t = null == r ? null : "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (null != t) { var e, n, i, u, a = [], f = !0, o = !1; try { if (i = (t = t.call(r)).next, 0 === l) { if (Object(t) !== t) return; f = !1; } else for (; !(f = (e = i.call(t)).done) && (a.push(e.value), a.length !== l); f = !0); } catch (r) { o = !0, n = r; } finally { try { if (!f && null != t.return && (u = t.return(), Object(u) !== u)) return; } finally { if (o) throw n; } } return a; } }
function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }
/* eslint-disable @typescript-eslint/no-unused-vars */
import { forwardRef, useCallback, useContext, useEffect, useImperativeHandle, useState } from 'react';
// import { Icon, Popover } from '@casstime/bricks';
import classNames from 'classnames';
import React from 'react';
import { ACTION, ACTION_TEXT, LANG, MAX_HISTORY_LEN, MENU_TYPE_ENUM, MENU_TYPE_TEXT, WORK_SPACE_ID, stayRemain } from "../constants";
import { EditorContext } from "../util";
import Popover from "./setting/Popover";
export var useHistory = function useHistory() {
  var _useContext = useContext(EditorContext),
    canvasInstanceRef = _useContext.canvasInstanceRef,
    canvasIsRender = _useContext.canvasIsRender,
    currentMenuRef = _useContext.currentMenuRef,
    initCanvasJson = _useContext.initCanvasJson,
    unSaveHistory = _useContext.unSaveHistory;
  var _useState = useState([]),
    _useState2 = _slicedToArray(_useState, 2),
    history = _useState2[0],
    setHistory = _useState2[1];
  var _useState3 = useState(''),
    _useState4 = _slicedToArray(_useState3, 2),
    currentId = _useState4[0],
    setCurrentId = _useState4[1];

  /**
   * action:操作类型
   * remove：是否移除最后一项
   * append：是否追加最新项目
   */
  var updateCanvasState = useCallback(function (action) {
    var remove = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var append = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    if (!canvasInstanceRef.current || unSaveHistory.current) {
      return;
    }
    var canvas = canvasInstanceRef.current;
    try {
      /** 列表为空 */
      var jsonData = canvas.toJSON(stayRemain);
      var canvasAsJson = JSON.stringify(jsonData);
      var id = Date.now() + '_' + Math.floor(Math.random() * 10000);
      var currentIndex = history.findIndex(function (_ref) {
        var _ref$id = _ref.id,
          hId = _ref$id === void 0 ? '' : _ref$id;
        return hId === currentId;
      });

      /**
       * （如果当前索引在中间，需要把索引后边的历史去掉）
       * 1、根据remove决定是否移除最后一条
       * 2、根据append决定是否追加一条
       * 3、根据MAX_HISTORY_LEN，最多存 MAX_HISTORY_LEN 条历史
       * */
      var newHistory = history.slice(0, currentIndex + 1 - +remove).concat(append || !remove && !append ? [{
        id: id,
        data: canvasAsJson,
        type: action !== ACTION.delete ? currentMenuRef.current : MENU_TYPE_ENUM.default,
        action: action
      }] : []);
      setHistory(newHistory.slice(Math.max(0, history.length - MAX_HISTORY_LEN)));
      setCurrentId(id);
    } catch (error) {
      console.error(error);
    }
  }, [history, currentId]);
  var objectModified = useCallback(function (e) {
    if (e.target.id === WORK_SPACE_ID || e.action === undefined && e.e) {
      return;
    }
    updateCanvasState(ACTION.modified);
  }, [updateCanvasState]);
  var objectAdd = useCallback(function (e) {
    if (e.action === undefined && e.e) {
      return;
    }
    updateCanvasState(ACTION.add);
  }, [updateCanvasState]);
  useEffect(function () {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    if (canvas && canvasIsRender) {
      canvas.on('object:modified', objectModified);
      canvas.on('object:added', objectAdd);
    }
    return function () {
      canvas.off('object:modified', objectModified);
      canvas.off('object:added', objectAdd);
    };
  }, [canvasInstanceRef, canvasIsRender, objectModified, objectAdd]);
  var renderNewCanvas = useCallback(function () {
    var _history$find;
    var id = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    if (id === currentId) {
      return;
    }
    var newCanvasStr = ((_history$find = history.find(function (_ref2) {
      var cId = _ref2.id;
      return cId === id;
    })) === null || _history$find === void 0 ? void 0 : _history$find.data) || initCanvasJson.current;
    unSaveHistory.current = true;
    canvas.loadFromJSON(newCanvasStr, function () {
      canvas.renderAll();
      unSaveHistory.current = false;
    });
    setCurrentId(id);
  }, [currentId, history]);

  /** 前进 */
  var handleRedo = useCallback(function () {
    if (!canvasInstanceRef.current || !history.length) {
      return;
    }
    var newId = '';
    if (!currentId) {
      var _history$;
      newId = (_history$ = history[0]) === null || _history$ === void 0 ? void 0 : _history$.id;
    } else {
      history.forEach(function (_ref3, index) {
        var id = _ref3.id;
        if (id === currentId) {
          var _history;
          newId = (_history = history[index + 1]) === null || _history === void 0 ? void 0 : _history.id;
        }
      });
    }
    renderNewCanvas(newId);
  }, [currentId, history]);

  /** 后退 */
  var handleUndo = useCallback(function () {
    if (!canvasInstanceRef.current || !history.length) {
      return;
    }
    var newId = '';
    history.forEach(function (_ref4, index) {
      var id = _ref4.id;
      if (id === currentId) {
        var _history2;
        newId = (_history2 = history[index - 1]) === null || _history2 === void 0 ? void 0 : _history2.id;
      }
    });
    renderNewCanvas(newId);
  }, [currentId, history]);
  return {
    history: history,
    currentId: currentId,
    renderNewCanvas: renderNewCanvas,
    handleRedo: handleRedo,
    handleUndo: handleUndo,
    updateCanvasState: updateCanvasState
  };
};

/** 历史 */
export var History = /*#__PURE__*/forwardRef(function (props, ref) {
  var _history3, _history4;
  var _useContext2 = useContext(EditorContext),
    _useContext2$lang = _useContext2.lang,
    lang = _useContext2$lang === void 0 ? LANG.en : _useContext2$lang;
  var _useHistory = useHistory(),
    history = _useHistory.history,
    currentId = _useHistory.currentId,
    renderNewCanvas = _useHistory.renderNewCanvas,
    handleRedo = _useHistory.handleRedo,
    handleUndo = _useHistory.handleUndo,
    updateCanvasState = _useHistory.updateCanvasState;
  useImperativeHandle(ref, function () {
    return {
      updateCanvasState: updateCanvasState
    };
  });
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: classNames('tie-image-editor_tool-item tie-image-editor_tool-history', {
      'tie-image-editor_tool-item--disabled': !history.length
    })
  }, /*#__PURE__*/React.createElement(Popover, {
    content: /*#__PURE__*/React.createElement("div", {
      className: "tie-image-editor-history_pop"
    }, !!history.length && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
      className: "tie-image-editor-history_pop-title"
    }, "\u5386\u53F2", /*#__PURE__*/React.createElement("span", {
      className: "tie-image-editor-history_pop-len"
    }, "\uFF08", history.length, "\uFF09")), /*#__PURE__*/React.createElement("div", {
      onClick: function onClick() {
        return renderNewCanvas('');
      },
      className: classNames('tie-image-editor-history_pop-item', {
        'tie-image-editor-history_pop-item--checked': !currentId
      })
    }, /*#__PURE__*/React.createElement("span", {
      className: "tie-image-editor-history_pop-item-num"
    }, "1."), "\u521D\u59CB\u5316", !currentId && /*#__PURE__*/React.createElement("i", {
      className: "tie-image-editor-history_pop-item-icon"
    }))), !history.length && /*#__PURE__*/React.createElement("span", null, "\u6682\u65E0\u5386\u53F2"), history.map(function (_ref5, index) {
      var _ref5$action = _ref5.action,
        action = _ref5$action === void 0 ? '' : _ref5$action,
        id = _ref5.id,
        type = _ref5.type;
      return /*#__PURE__*/React.createElement("div", {
        key: id,
        onClick: function onClick() {
          return renderNewCanvas(id);
        },
        className: classNames('tie-image-editor-history_pop-item', {
          'tie-image-editor-history_pop-item--checked': id === currentId
        })
      }, /*#__PURE__*/React.createElement("span", {
        className: "tie-image-editor-history_pop-item-num"
      }, index + 2, "."), ACTION_TEXT[action], !!MENU_TYPE_TEXT[type] && MENU_TYPE_TEXT[type][lang], id === currentId && /*#__PURE__*/React.createElement("i", {
        className: "tie-image-editor-history_pop-item-icon"
      }));
    })),
    placement: "bottom",
    className: "tie-image-editor-history_target"
  }, /*#__PURE__*/React.createElement("i", {
    className: classNames('tie-image-editor_icon')
  }))), /*#__PURE__*/React.createElement("div", {
    className: classNames('tie-image-editor_tool-item tie-image-editor_tool-redo', {
      'tie-image-editor_tool-item--disabled': !history.length || currentId === ((_history3 = history[history.length - 1]) === null || _history3 === void 0 ? void 0 : _history3.id)
    })
  }, /*#__PURE__*/React.createElement(Popover, {
    content: MENU_TYPE_TEXT.redo[lang],
    placement: "top"
  }, /*#__PURE__*/React.createElement("i", {
    className: classNames('tie-image-editor_icon'),
    onClick: !history.length || currentId === ((_history4 = history[history.length - 1]) === null || _history4 === void 0 ? void 0 : _history4.id) ? undefined : handleRedo
  }))), /*#__PURE__*/React.createElement("div", {
    className: classNames('tie-image-editor_tool-item tie-image-editor_tool-undo', {
      'tie-image-editor_tool-item--disabled': !history.length || !currentId
    })
  }, /*#__PURE__*/React.createElement(Popover, {
    content: MENU_TYPE_TEXT.undo[lang],
    placement: "top"
  }, /*#__PURE__*/React.createElement("i", {
    className: classNames('tie-image-editor_icon'),
    onClick: !history.length || !currentId ? undefined : handleUndo
  }))));
});
History.displayName = 'History';