function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(arg) { var key = _toPrimitive(arg, "string"); return _typeof(key) === "symbol" ? key : String(key); }
function _toPrimitive(input, hint) { if (_typeof(input) !== "object" || input === null) return input; var prim = input[Symbol.toPrimitive]; if (prim !== undefined) { var res = prim.call(input, hint || "default"); if (_typeof(res) !== "object") return res; throw new TypeError("@@toPrimitive must return a primitive value."); } return (hint === "string" ? String : Number)(input); }
import classNames from 'classnames';
import React, { useCallback, useContext, useEffect, useRef } from 'react';
import { fabric } from 'fabric';
import { LANG, MENU_TYPE_ENUM, MENU_TYPE_TEXT, paintConfig } from "../constants";
import { EditorContext } from "../util";
import MosaicPop from "./setting/MosaicPop";
import Popover from "./setting/Popover";
var blockSize = paintConfig.mosaicSizes[0];
var mosaicify = function mosaicify(imageData) {
  var data = imageData.data;
  var iLen = imageData.height;
  var jLen = imageData.width;
  var index;
  var i;
  var j;
  var r;
  var g;
  var b;
  var a;
  var _i;
  var _j;
  var _iLen;
  var _jLen;
  for (i = 0; i < iLen; i += blockSize) {
    for (j = 0; j < jLen; j += blockSize) {
      index = i * 4 * jLen + j * 4;
      r = data[index];
      g = data[index + 1];
      b = data[index + 2];
      a = data[index + 3];
      _iLen = Math.min(i + blockSize, iLen);
      _jLen = Math.min(j + blockSize, jLen);
      for (_i = i; _i < _iLen; _i++) {
        for (_j = j; _j < _jLen; _j++) {
          index = _i * 4 * jLen + _j * 4;
          data[index] = r;
          data[index + 1] = g;
          data[index + 2] = b;
          data[index + 3] = a;
          /*
          data[index] = 0;
          data[index + 1] = 0;
          data[index + 2] = 0;
          */
        }
      }
    }
  }
};

var useMosaic = function useMosaic() {
  var _useContext = useContext(EditorContext),
    canvasInstanceRef = _useContext.canvasInstanceRef,
    canvasIsRender = _useContext.canvasIsRender,
    currentMenu = _useContext.currentMenu,
    currentMenuRef = _useContext.currentMenuRef,
    setCurrentMenu = _useContext.setCurrentMenu;
  var mosaicBrush = useRef(null);
  var initMosaic = useCallback(function () {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    mosaicBrush.current = new fabric.PatternBrush(canvas);
    mosaicBrush.current.getPatternSrc = function () {
      // 创立一个暂存 canvas 来绘製要画的图案
      var cropping = {
        left: 0,
        top: 0,
        width: canvas.width,
        height: canvas.height
      };
      var imageCanvas = canvas.toCanvasElement(1, cropping);
      var imageCtx = imageCanvas.getContext('2d');
      var imageData = imageCtx.getImageData(0, 0, imageCanvas.width, imageCanvas.height);
      mosaicify(imageData);
      imageCtx.putImageData(imageData, 0, 0);
      var patternCanvas = fabric.document.createElement('canvas'); // 这里的ceateElement一定要使用fabric内置的方法
      var patternCtx = patternCanvas.getContext('2d');
      patternCanvas.width = canvas.width || 0;
      patternCanvas.height = canvas.height || 0;
      patternCtx.drawImage(imageCanvas, 0, 0, imageCanvas.width, imageCanvas.height, cropping.left, cropping.top, cropping.width, cropping.height);
      return patternCanvas;
    };
  }, []);
  useEffect(function () {
    if (canvasIsRender) {
      initMosaic();
    }
  }, [canvasIsRender]);
  var handleMosaicTrigger = useCallback(function () {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    if (currentMenuRef.current !== MENU_TYPE_ENUM.mosaic) {
      /** 启动马赛克 */
      canvas.isDrawingMode = true;
      canvas.freeDrawingBrush = mosaicBrush.current;
      var brush = canvas.freeDrawingBrush;
      if (brush.getPatternSrc) {
        brush.source = brush.getPatternSrc.call(brush);
      }
      brush.width = 30;
      brush.shadow = new fabric.Shadow({
        blur: 0,
        offsetX: 0,
        offsetY: 0,
        affectStroke: true
      });
    } else {
      canvas.isDrawingMode = false;
    }
    setCurrentMenu(currentMenuRef.current === MENU_TYPE_ENUM.mosaic ? MENU_TYPE_ENUM.default : MENU_TYPE_ENUM.mosaic);
  }, [initMosaic]);
  var handleSizeChange = function handleSizeChange(size) {
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }
    blockSize = size;
    var brush = canvas.freeDrawingBrush;
    if (brush.getPatternSrc) {
      brush.source = brush.getPatternSrc.call(brush);
    }
  };
  return {
    currentMenu: currentMenu,
    handleMosaicTrigger: handleMosaicTrigger,
    handleSizeChange: handleSizeChange
  };
};

/**
 * 马赛克
 */
var Mosaic = function Mosaic() {
  var _useContext2 = useContext(EditorContext),
    _useContext2$lang = _useContext2.lang,
    lang = _useContext2$lang === void 0 ? LANG.en : _useContext2$lang;
  var _useMosaic = useMosaic(),
    currentMenu = _useMosaic.currentMenu,
    handleMosaicTrigger = _useMosaic.handleMosaicTrigger,
    handleSizeChange = _useMosaic.handleSizeChange;
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: classNames('tie-image-editor_tool-item tie-image-editor_tool-mosaic', _defineProperty({}, 'tie-image-editor_tool-item--checked', currentMenu === MENU_TYPE_ENUM.mosaic))
  }, /*#__PURE__*/React.createElement(MosaicPop, {
    open: currentMenu === MENU_TYPE_ENUM.mosaic,
    onChange: handleSizeChange
  }, /*#__PURE__*/React.createElement(Popover, {
    content: MENU_TYPE_TEXT.mosaic[lang],
    placement: "top"
  }, /*#__PURE__*/React.createElement("i", {
    className: "tie-image-editor_icon",
    onClick: handleMosaicTrigger
  })))));
};
export default Mosaic;