import { fabric } from 'fabric';
import React, { useContext } from 'react';
import { ACTION, LANG, MENU_TYPE_ENUM, MENU_TYPE_TEXT } from "../constants";
import { EditorContext } from "../util";
import Popover from "./setting/Popover";
var useReset = function useReset() {
  var _useContext = useContext(EditorContext),
    setCurrentMenu = _useContext.setCurrentMenu,
    currentMenuRef = _useContext.currentMenuRef,
    canvasInstanceRef = _useContext.canvasInstanceRef,
    historyRef = _useContext.historyRef;
  var handleReset = function handleReset() {
    if (currentMenuRef.current !== MENU_TYPE_ENUM.reset) {
      setCurrentMenu(MENU_TYPE_ENUM.reset);
    }
    var canvas = canvasInstanceRef.current;
    if (!canvas) {
      return;
    }

    /** 清空除了背景之外的图形 */
    canvas.getObjects().forEach(function (o) {
      if (o !== canvas.backgroundImage) {
        canvas.remove(o);
      }
    });

    /** 将缩放也重置 */
    var center = canvas.getCenter();
    canvas.zoomToPoint(new fabric.Point(center.left, center.top), 1);

    /** 将画布拖回初始值位置 */
    var viewportTransform = canvas.viewportTransform;
    viewportTransform[4] = 0;
    viewportTransform[5] = 0;
    canvas.setViewportTransform(viewportTransform);
    canvas.renderAll();

    /** 保存历史 */
    historyRef.current.updateCanvasState(ACTION.add);
  };
  return {
    handleReset: handleReset
  };
};

/** 重置 */
export var Reset = function Reset() {
  var _useContext2 = useContext(EditorContext),
    _useContext2$lang = _useContext2.lang,
    lang = _useContext2$lang === void 0 ? LANG.en : _useContext2$lang;
  var _useReset = useReset(),
    handleReset = _useReset.handleReset;
  return /*#__PURE__*/React.createElement("div", {
    className: "tie-image-editor_tool-item tie-image-editor_tool-reset"
  }, /*#__PURE__*/React.createElement(Popover, {
    content: MENU_TYPE_TEXT.reset[lang],
    placement: "top"
  }, /*#__PURE__*/React.createElement("i", {
    className: "tie-image-editor_icon",
    onClick: handleReset
  })));
};