/* eslint-disable no-param-reassign */
import { fabric } from 'fabric';

/** 自定义箭头 */
var Arrow = fabric.util.createClass(fabric.Line, {
  type: 'arrow',
  superType: 'drawing',
  initialize: function initialize(points, options) {
    if (!points) {
      var _options = options,
        x1 = _options.x1,
        x2 = _options.x2,
        y1 = _options.y1,
        y2 = _options.y2;
      points = [x1, y1, x2, y2];
    }
    options = options || {};
    this.callSuper('initialize', points, options);
  },
  _render: function _render(ctx) {
    this.callSuper('_render', ctx);
    ctx.save();
    var xDiff = this.x2 - this.x1;
    var yDiff = this.y2 - this.y1;
    var angle = Math.atan2(yDiff, xDiff);
    ctx.translate((this.x2 - this.x1) / 2, (this.y2 - this.y1) / 2);
    ctx.rotate(angle);
    ctx.beginPath();
    /** 箭头上的三角形大小 */
    ctx.moveTo(this.strokeWidth * 2, 0);
    ctx.lineTo(-this.strokeWidth * 2, this.strokeWidth * 2);
    ctx.lineTo(-this.strokeWidth * 2, -this.strokeWidth * 2);
    ctx.closePath();
    ctx.fillStyle = this.stroke;
    ctx.fill();
    ctx.restore();
  }
});
Arrow.fromObject = function (options, callback) {
  var x1 = options.x1,
    x2 = options.x2,
    y1 = options.y1,
    y2 = options.y2;
  return callback(new fabric.Arrow([x1, y1, x2, y2], options));
};
export default Arrow;